@@ -26,7 +26,6 @@ Examples
 
 The following example creates and runs a listener::
 
-    import sys
     import logging
     from socket import getfqdn
     from pywbem import WBEMListener
@@ -34,20 +33,21 @@ The following example creates and runs a listener::
     def process_indication(indication, host):
         '''This function gets called when an indication is received.'''
 
-        print("Received CIM indication from {host}: {ind!r}". \\
-            format(host=host, ind=indication))
+        print("Received CIM indication from {host}: {ind!r}".
+              format(host=host, ind=indication))
 
     def main():
 
         # Configure logging of the listener via the Python root logger
-        logging.basicConfig(filename='listener.log', level=logging.WARNING,
-            format='%(asctime)s - %(levelname)s - %(message)s)
+        logging.basicConfig(
+            filename='listener.log', level=logging.WARNING,
+            format='%(asctime)s - %(levelname)s - %(message)s')
 
         certkeyfile = 'listener.pem'
 
-        listener = WBEMListener(host=getfqdn()
-                                http_port=5988,
-                                https_port=5989,
+        listener = WBEMListener(host=getfqdn(),
+                                http_port=5990,
+                                https_port=5991,
                                 certfile=certkeyfile,
                                 keyfile=certkeyfile)
         listener.add_callback(process_indication)
@@ -57,7 +57,7 @@ The following example creates and runs a listener::
 
             # process_indication() will be called for each received indication
 
-            ... # wait for some condition to end listening
+            . . .  # wait for some condition to end listening
 
         finally:
             listener.stop()
@@ -104,9 +104,9 @@ back to the indication sender, at the :attr:`py:logging.INFO` logging level.
 
 In addition, it will log errors at the :attr:`py:logging.ERROR` logging level.
 
-Since Python 2.7, the Python root logger will by default (i.e. when not being
-configured) print log records of logging level :attr:`py:logging.WARNING` or
-greater to `sys.stderr`. So the indication and response interactions will not
+Starting with Python 2.7, the Python root logger will by default (i.e. when not
+being configured) print log records of logging level :attr:`py:logging.WARNING`
+or greater to `sys.stderr`. So the indication and response interactions will not
 be printed by default, but any errors logged at the :attr:`py:logging.ERROR`
 logging level will be printed by default.
 
@@ -117,7 +117,7 @@ This follows best practices recommended in `Configuring logging for a library
 and in several articles, for example in `this article
 <http://pieces.openpolitics.com/2012/04/python-logging-best-practices/>`_.
 Because this warning is no longer issued on Python 3.4 and higher, pywbem
-adds a null handler only on Python 2.6 and 2.7.
+adds a null handler only on Python 2.7.
 
 
 .. _`WBEMListener class`:
@@ -305,22 +305,17 @@ class ListenerRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
         if tq_list is not None:
             for token, quality in tq_list:
                 quality = 1 if quality == '' else float(quality)
-                if token.lower() == 'identity':
+                if token.lower() in ('identity', '*'):
                     identity_found = True
                     if quality > 0:
                         identity_acceptable = True
                     break
-            if not identity_found:
-                for token, quality in tq_list:
-                    quality = 1 if quality == '' else float(quality)
-                    if token == '*' and quality > 0:
-                        identity_acceptable = True
-                        break
-        if not identity_acceptable:
+        if not identity_found or not identity_acceptable:
             self.send_http_error(
                 406, 'header-mismatch',
                 _format("Invalid Accept-Encoding header value: {0} "
-                        "(need Identity to be acceptable)", accept_encoding))
+                        "(need Identity with quality > 0 to be acceptable)",
+                        accept_encoding))
             return
 
         # Accept-Language header check described in DSP0200.
