@@ -71,8 +71,11 @@ class BaseProvider(object):
 
     def __repr__(self):
         return _format(
-            "(self.__class__.__name__)("
-            "cimrepository={s.cimrepository}, ",
+            "{s.__class__.__name__}("
+            "provider_type={s.provider_type!A}, "
+            "provider_classnames={s.provider_classnames!A}, "
+            "_interop_namespace_names={s._interop_namespace_names!A}, "
+            "cimrepository=...)",
             s=self)
 
     @property
